//
//  KeyboardScene.cpp
//  layer_keyboard
//
//  Created by Reza Ghobadinic on 2019-09-08.
//

#include "KeyboardScene.h"
#include <iostream>

USING_NS_CC;

Scene* KeyboardScene::createScene()
{
    auto scene = Scene::create();
    auto layer = KeyboardScene::create();
    
    scene->addChild(layer);
    
    return scene;
}

bool KeyboardScene::init()
{
    auto visibleSize = Director::getInstance()->getVisibleSize();
    Vec2 origin = Director::getInstance()->getVisibleOrigin();
    
    x = origin.x + visibleSize.width / 2;
    y = origin.y + visibleSize.height / 2;
    
    auto sprite = Sprite::create("Box.png");
    sprite->setPosition(x, y);
    this->addChild(sprite, 0, 1);
    
    auto listener = EventListenerKeyboard::create();
    
    listener->onKeyPressed  = CC_CALLBACK_2(KeyboardScene::onKeyPressed, this);
    listener->onKeyReleased = CC_CALLBACK_2(KeyboardScene::onKeyReleased, this);
    
    _eventDispatcher->addEventListenerWithSceneGraphPriority(listener, this);
    
    this->scheduleUpdate();
    
    return true;
}

void KeyboardScene::onKeyPressed(EventKeyboard::KeyCode keyCode, Event* event)
{
    if (keys.find(keyCode) == keys.end())
        keys[keyCode] = std::chrono::system_clock::now();
    
    std::cout << "Number of Keys: " << keys.size() << std::endl;
}

void KeyboardScene::onKeyReleased(EventKeyboard::KeyCode keyCode, Event* event)
{
    if (keys.find(keyCode) != keys.end())
    {
        std::chrono::duration<double> elapsed_seconds = std::chrono::system_clock::now() - keys[keyCode];
        std::cout << "Key code " << int(keyCode) << " was down for " << elapsed_seconds.count() << "s" << std::endl;
    }
    
    keys.erase(keyCode);
    
    std::cout << "Number of Keys: " << keys.size() << std::endl;
}

void KeyboardScene::update(float delta)
{
    float translate = 10.0 * delta;
    
    if (keys.find(EventKeyboard::KeyCode::KEY_LEFT_ARROW) != keys.end())
        x -= translate;
    
    if (keys.find(EventKeyboard::KeyCode::KEY_RIGHT_ARROW) != keys.end())
        x += translate;
    
    if (keys.find(EventKeyboard::KeyCode::KEY_UP_ARROW) != keys.end())
        y += translate;
    
    if (keys.find(EventKeyboard::KeyCode::KEY_DOWN_ARROW) != keys.end())
        y -= translate;
    
    auto sprite = this->getChildByTag(1);
    sprite->setPosition(x, y);
}
